@@ -23,7 +23,7 @@ import time
 import fcntl
 
 from utils.common import _error
-from cl_template import templateFormat, TemplatesError
+from cl_template import FormatFactory, TemplatesError
 
 from calculate.lib.cl_lang import setLocalTranslate
 
@@ -31,7 +31,7 @@ _ = lambda x: x
 setLocalTranslate('cl_lib3', sys.modules[__name__])
 
 
-class iniParser(_error, templateFormat):
+class iniParser(_error):
     """Класс для работы с ini файлами
 
     """
@@ -47,6 +47,7 @@ class iniParser(_error, templateFormat):
         self.readOnly = False
         self.text = text
         self.locked = False
+        self.formatFactory = FormatFactory(self)
 
     def joinText(self, iniObj, xmlNewDoc):
         """Объединяет два документа"""
@@ -165,7 +166,7 @@ class iniParser(_error, templateFormat):
             return False
         if type(strHeader) in (tuple, list):
             # формат plasma
-            classObj = self.getClassObj("plasma")
+            classObj = self.formatFactory.getClassObj("plasma")
         else:
             if nameFomat == "plasma":
                 self.setError(_("Trying to write a variable of 'samba' "
@@ -173,15 +174,15 @@ class iniParser(_error, templateFormat):
                               % self.iniFile)
                 return False
             # формат samba
-            classObj = self.getClassObj("samba")
+            classObj = self.formatFactory.getClassObj("samba")
         # создаем объект
         # и записываем в него содержимое ini-файла
-        objIni = classObj(textIni)
+        objIni = classObj(textIni, self)
         # создаем текст из строки заголовка и
         # словаря переменных области
         txtConfig = objIni.createTxtConfig(strHeader, dictVar)
         # создаем объект и записываем в него текст
-        objIniAdd = classObj(txtConfig)
+        objIniAdd = classObj(txtConfig, self)
         # объединяем объекты для получения результирующего текста
         objIni.join(objIniAdd)
         # получаем текст
@@ -218,7 +219,8 @@ class iniParser(_error, templateFormat):
             # В файле есть данные
             if not self.isEmptyFile(textIni):
                 try:
-                    objIni = self.getClassObj("plasma")(textIni)
+                    objIni = self.formatFactory.getClassObj(
+                        "plasma")(textIni, self)
                 except Exception:
                     self.setError(_("Incorrect file format") + _(": ") + \
                                   self.iniFile)
@@ -231,7 +233,8 @@ class iniParser(_error, templateFormat):
                         self.checkIni = "plasma"
                         break
                 if self.checkIni == "samba":
-                    objIni = self.getClassObj("samba")(textIni)
+                    objIni = self.formatFactory.getClassObj(
+                        "samba")(textIni, self)
                     xmlBody = objIni.docObj.getNodeBody()
                     if not xmlBody.firstChild:
                         self.checkIni = False
@@ -269,7 +272,7 @@ class iniParser(_error, templateFormat):
         formatPlasma = False
         if type(strHeader) in (tuple, list):
             # формат plasma
-            classObj = self.getClassObj("plasma")
+            classObj = self.formatFactory.getClassObj("plasma")
             formatPlasma = True
         else:
             if nameFomat == "plasma":
@@ -278,9 +281,9 @@ class iniParser(_error, templateFormat):
                               % self.iniFile)
                 return False
             # формат samba
-            classObj = self.getClassObj("samba")
+            classObj = self.formatFactory.getClassObj("samba")
         # создаем объект и записываем в него содержимое ini-файла
-        objIni = classObj(textIni)
+        objIni = classObj(textIni, self)
         # получаем ноду body
         xmlBody = objIni.docObj.getNodeBody()
         flagFound, xmlBody = self.getLastNode(objIni, xmlBody, strHeader,
@@ -335,7 +338,7 @@ class iniParser(_error, templateFormat):
         formatPlasma = False
         if type(strHeader) in (tuple, list):
             # формат plasma
-            classObj = self.getClassObj("plasma")
+            classObj = self.formatFactory.getClassObj("plasma")
             formatPlasma = True
         else:
             if nameFomat == "plasma":
@@ -344,9 +347,9 @@ class iniParser(_error, templateFormat):
                               % self.iniFile)
                 return False
             # формат samba
-            classObj = self.getClassObj("samba")
+            classObj = self.formatFactory.getClassObj("samba")
         # создаем объект типа samba и записываем в него содержимое ini-файла
-        objIni = classObj(textIni)
+        objIni = classObj(textIni, self)
         # получаем ноду body
         xmlBody = objIni.docObj.getNodeBody()
         flagFound, xmlBody = self.getLastNode(objIni, xmlBody, strHeader,
@@ -378,11 +381,11 @@ class iniParser(_error, templateFormat):
             return False
         if nameFomat == "samba":
             # создаем объект типа samba и записываем в него содержимое ini-файла
-            objIni = self.getClassObj("samba")(textIni)
+            objIni = self.formatFactory.getClassObj("samba")(textIni, self)
         elif nameFomat == "plasma":
             # создаем объект типа plasma и записываем в него содержимое
             # ini-файла
-            objIni = self.getClassObj("plasma")(textIni)
+            objIni = self.formatFactory.getClassObj("plasma")(textIni, self)
         else:
             return []
         xmlNodes = objIni.docObj.getAllAreas()
