@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright 2008-2016 Mir Calculate. http://www.calculate-linux.org
+# Copyright 2008-2017 Mir Calculate. http://www.calculate-linux.org
 #
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
@@ -30,8 +30,10 @@ import re
 import sys
 from itertools import *
 import tarfile
+import tools
 import fcntl
-from tools import SingletonParam
+from tools import ignore
+from glob import glob
 
 from contextlib import contextmanager
 import signal
@@ -111,6 +113,8 @@ class PipeProcess(StdoutableProcess):
     def get_stdout(self):
         return PIPE
 
+class ProcessTimeout(FilesError):
+    pass
 
 class process(StdoutableProcess):
     """Execute system command by Popen
@@ -158,6 +162,7 @@ class process(StdoutableProcess):
         self.envdict = kwarg.get("envdict", os.environ.copy())
         self.envdict["LANG"] = kwarg.get('lang', 'C')
         self.langc = "langc" in kwarg
+        self.timeout = kwarg.get("timeout", None)
 
         self.stderr = kwarg.get("stderr", PIPE)
         self.cwd = kwarg.get("cwd", None)
@@ -194,8 +199,11 @@ class process(StdoutableProcess):
     def write(self, data):
         """Write to process stdin"""
         self._open()
-        self.pipe.stdin.write(data)
-        self.pipe.stdin.flush()
+        try:
+            self.pipe.stdin.write(data)
+            self.pipe.stdin.flush()
+        except IOError as e:
+            raise FilesError(str(e))
 
     def close(self):
         """Close stdin"""
@@ -203,8 +211,10 @@ class process(StdoutableProcess):
             self.pipe.poll()
             if self.pipe.stdin:
                 self.pipe.stdin.close()
+                self.pipe.stdin = None
         if self.stdin:
             self.stdin.close()
+            self.stdin = None
 
     def readerr(self):
         self.read()
@@ -223,8 +233,13 @@ class process(StdoutableProcess):
                 _stdout = self.pipe.stdout.fileno()
                 _stderr = self.pipe.stderr.fileno()
                 reads = [_stdout, _stderr]
+                self.pipe.stdin.close()
+                self.pipe.stdin = None
                 while True:
-                    ret = select.select(reads, [], [])
+                    ret = select.select(reads, [], [], self.timeout)
+                    if not ret[0]:
+                        self.kill()
+                        raise ProcessTimeout()
                     for fd in ret[0]:
                         if fd == _stdout:
                             s = self.pipe.stdout.readline()
@@ -262,7 +277,11 @@ class process(StdoutableProcess):
         try:
             self._open()
             if self.cacheresult is None:
-                self.cacheresult, self.cacheerr = self.pipe.communicate()
+                if self.timeout is None:
+                    self.cacheresult, self.cacheerr = self.pipe.communicate()
+                else:
+                    for line in self.readByLine():
+                        pass
             return self.cacheresult
         except KeyboardInterrupt:
             self.kill()
@@ -614,7 +633,6 @@ def check_rw(dn):
     :param dn:
     :return:
     """
-
     def _random_string(l=10):
         return "".join(
             random.choice(string.ascii_letters) for i in xrange(0, l))
@@ -630,6 +648,17 @@ def check_rw(dn):
         return False
 
 
+def quite_unlink(fn):
+    """
+    Проверить существует ли файл и удалить его, не поднимать Exception
+    если не удалось удалить
+    :param fn:
+    :return:
+    """
+    with ignore(OSError):
+        if path.lexists(fn):
+            os.unlink(fn)
+
 def removeFileWithEmptyDirectory(rmName, stopDirectory="/"):
     """
     Удалить файл и все пустые родительские директории
@@ -647,7 +676,7 @@ def removeFileWithEmptyDirectory(rmName, stopDirectory="/"):
                                          stopDirectory=stopDirectory)
 
 
-def getRunCommands(not_chroot=False, chroot=None, uid=None):
+def getRunCommands(not_chroot=False, chroot=None, uid=None, withpid=False):
     """List run program"""
 
     def getCmd(procNum):
@@ -673,76 +702,16 @@ def getRunCommands(not_chroot=False, chroot=None, uid=None):
 
     if not os.access('/proc', os.R_OK):
         return []
-    return filter(None,
-                  map(getCmd,
-                      filter(lambda x: x.isdigit(),
-                             listDirectory('/proc'))))
-
-
-def isMount(dn):
-    """
-    Возвращает путь примонтированного dn
-    """
-
-    def find_names(old_dn):
-        dn = path.abspath(old_dn)
-        yield dn
-        if dn.startswith('/dev'):
-            info = device.getUdevDeviceInfo(name=dn)
-            if 'DM_VG_NAME' in info and 'DM_LV_NAME' in info:
-                yield '/dev/mapper/{vg}-{lv}'.format(vg=info['DM_VG_NAME'],
-                                                     lv=info['DM_LV_NAME'])
-
-    def get_overlay_mounts(line):
-        mounts = line.split(' ')
-        yield mounts[1]
-        for dn in re.findall("(?:lowerdir=|upperdir=|workdir=)([^,]+)",
-                             mounts[3]):
-            yield dn
-
-    find_data = set(find_names(dn))
-    ret = ""
-    for line in readLinesFile('/etc/mtab'):
-        if " overlay " not in line:
-            if " " in line:
-                mounts = set(line.split(' ')[:2])
-                if mounts & find_data:
-                    ret = (mounts - find_data).pop()
-        else:
-            mounts = set(get_overlay_mounts(line))
-            dest = line.split(' ')[1]
-            if mounts & find_data:
-                if dest in find_data:
-                    ret = "overlay"
-                else:
-                    return dest
-    return ret
-
-
-def commonPath(*paths):
-    """Return common path from list of paths"""
-    paths = map(lambda x: path.normpath(x).split('/'), paths)
-    res = map(lambda x: x[0],
-              filter(lambda x: filter(lambda y: x[0] == y, x[1:]), zip(*paths)))
-    return "/".join(res)
-
-
-def childMounts(pathname):
-    """Get all mount points which contain path"""
-    if pathname != "none":
-        absPath = path.abspath(pathname)
+    if withpid:
+        return filter(lambda x: x[1],
+                      map(lambda x:(x,getCmd(x)),
+                          filter(lambda x: x.isdigit(),
+                                 listDirectory('/proc'))))
     else:
-        absPath = pathname
-    mtabFile = '/etc/mtab'
-    if not os.access(mtabFile, os.R_OK):
-        return ""
-    return reduce(lambda x, y: x + [y],
-                  filter(lambda x: commonPath(absPath, x[0]) == absPath or \
-                                   commonPath(absPath, x[1]) == absPath,
-                         map(lambda x: [x[0], x[1]],
-                             map(lambda x: x.split(" "),
-                                 open(mtabFile)))),
-                  [])
+        return filter(None,
+                      map(getCmd,
+                          filter(lambda x: x.isdigit(),
+                                 listDirectory('/proc'))))
 
 
 def pathJoin(*paths):
@@ -924,7 +893,6 @@ class GetProgPath(object):
 
 getProgPath = GetProgPath()
 
-
 def checkUtils(*utils):
     """Check utils, exit if it not found and return fullpath"""
     retval = []
@@ -955,82 +923,6 @@ def getCmdLineParam(paramName):
         return ""
 
 
-from device import getUUIDDict
-import device
-
-
-class MountHelper(object):
-    """Data reader for fstab"""
-    data_file = '/etc/fstab'
-    NAME, DIR, TYPE, OPTS, FREQ, PASSNO = range(0, 6)
-
-    def __init__(self, data_file=None, devs=()):
-        if data_file:
-            self.data_file = data_file
-        self.cache = []
-        self.rotateCache = []
-        self.dictUUID = getUUIDDict(devs=devs)
-        self.rebuildCache()
-
-    def rebuildCache(self):
-        """Rebuild cache from fstab file"""
-
-        def setlen(ar):
-            return ar[:6] + [""] * (6 - len(ar))
-
-        self.cache = \
-            map(lambda x: setlen(map(lambda y: y.strip(), x.split())),
-                filter(lambda x: x.strip() and not x.lstrip().startswith("#"),
-                       open(self.data_file, 'r').read().split('\n')))
-        for data in self.cache:
-            convertDev = lambda x: path.realpath(x) if x.startswith('/') else x
-            data[0] = device.getUdevDeviceInfo(
-                name=convertDev(self.dictUUID.get(data[0], data[0]))
-            ).get('DEVNAME', data[0])
-            data[1] = data[1] if data[2] != "swap" else "swap"
-        self.rotateCache = zip(*self.cache)
-
-    def getBy(self, what=DIR, where=NAME, _in=None, eq=None,
-              contains=None, noteq=None, allentry=False):
-        """Get data from fstab"""
-        if eq is not None:
-            filterfunc = lambda x: x[where] == eq
-        elif _in is not None:
-            filterfunc = lambda x: x[where] in _in
-        elif contains is not None:
-            filterfunc = lambda x: contains in x[where]
-        else:
-            filterfunc = lambda x: x[where] != noteq
-        res = map(lambda x: x[what], filter(filterfunc, self.cache))
-        if allentry:
-            return res
-        else:
-            return "" if not res else res[-1]
-
-    def getFields(self, *fields):
-        """Get all data by specifie fields"""
-        return zip(*reduce(lambda x, y: x + [self.rotateCache[y]], fields, []))
-
-    def isExists(self, what=DIR, eq=None, noteq=None):
-        """Field with condition exist in fstab"""
-        if not eq is None:
-            filterfunc = lambda x: x[what] == eq
-        else:
-            filterfunc = lambda x: x[what] != noteq
-        return bool(filter(filterfunc, self.cache))
-
-
-class FStab(MountHelper):
-    """Data reader for fstab"""
-    __metaclass__ = SingletonParam
-    data_file = '/etc/fstab'
-
-
-class Mounts(MountHelper):
-    """Data reader for fstab"""
-    data_file = '/etc/mtab'
-
-
 def tarLinks(rootpath, archpath, skip=()):
     """
     Поместить все симлинки в архив
@@ -1383,33 +1275,6 @@ class scanDirectory(object):
         return ret
 
 
-class DiskSpaceError(Exception):
-    pass
-
-
-class DiskSpace(object):
-    def __init__(self):
-        self.df_cmd = getProgPath('/bin/df')
-
-    def get_free(self, dev=None, dn=None):
-        if dev:
-            mp = isMount(dev)
-            if not mp:
-                raise DiskSpaceError(_("Device %s must be mounted") % dev)
-            dn = dev
-        p = process(self.df_cmd, dn, "-B1")
-        if p.success():
-            data = p.read().strip()
-            lines = data.split('\n')
-            if len(lines) >= 2:
-                cols = filter(None, lines[1].split())
-                if len(cols) == 6:
-                    return int(cols[3])
-            raise DiskSpaceError(_("Wrong df output:\n%s") % data)
-        else:
-            raise DiskSpaceError(str(p.readerr()))
-
-
 def calculate_exclude(base_dn, exclude=(), include=()):
     """
     Вычисление исключений с использованием фильтров include
@@ -1554,7 +1419,6 @@ def tar_unpack(archfile, target_dn):
         with tarfile.open(archfile, "r:bz2") as tar:
             for item in tar:
                 tar.extract(item, target_dn)
-                print item.path, item.mtime
                 os.utime(path.join(target_dn, item.path),
                          (item.mtime, item.mtime))
     except (tarfile.TarError, IOError, OSError) as e:
@@ -1569,7 +1433,7 @@ class RsyncOptions(object):
     FilesFromStdin = "--files-from=-"
 
     @classmethod
-    def Chown(self, uid, gid):
+    def Chown(cls, uid, gid):
         return "--chown={0}:{1}".format(uid, gid)
 
     # опции используемые при "скрывании пакета"
@@ -1768,3 +1632,43 @@ class DirectoryRCS(object):
                 raise RCSError(_("Failed to move data: %s") % str(e))
 
 
+class RealFs(tools.GenericFs):
+    """
+    Файловая система, предоставляющая доступ к реальным данным
+    """
+    def __init__(self, prefix="/"):
+        self.prefix = prefix
+        if prefix == "/":
+            self.remove_prefix = lambda x: x
+        else:
+            self.remove_prefix = self._remove_prefix
+
+    def _remove_prefix(self, dn):
+        return dn[:len(self.prefix)]
+
+    def _path(self, dn):
+        return pathJoin(self.prefix, dn)
+
+    def exists(self, fn):
+        return path.lexists(self._path(fn))
+
+    def read(self, fn):
+        return readFile(self._path(fn))
+
+    def glob(self, dn):
+        for fn in glob(self._path(dn)):
+            yield self.remove_prefix(fn)
+
+    def realpath(self, fn):
+        return self.remove_prefix(path.realpath(fn))
+
+    def write(self, fn, data):
+        with writeFile(fn) as f:
+            f.write(data)
+
+    def listdir(self, dn, fullpath=False):
+        if fullpath:
+            return [self.remove_prefix(x)
+                    for x in listDirectory(dn, fullPath=True)]
+        else:
+            return listDirectory(dn, fullPath=False)
